/*
-------------------------------------------------------------------
SUDOKU SENSEI 1.04: a Sudoku Explainer Engine
Copyright (C) 2005-2007  Martin Knoblauch

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
MA  02110-1301, USA.

Contact the author: comocomocomo AT users.sourceforge.net
Latest versions: http://sourceforge.net/projects/sudoku-sensei
-------------------------------------------------------------------

    sudoku-solver.c
*/

#include <stdio.h>
#include <stdlib.h>
#include <memory.h>

#include "sudoku-pr.h"

#ifdef INLINE
#undef INLINE
#endif
#define INLINE

/*///////////////////////////////////////////////////////////////// */

static INLINE void ProcessDirtyCell (t_board * b);

static INLINE int ProcessDirtyGroup (t_board * b,
                                     char apply,
                                     char exhaustive,
                                     volatile char * abort);

static INLINE void RemovePossibleValuesFromCell (t_board * b,
                                                 t_cell * c,
                                                 int m);

static INLINE void SearchCandidateCellForGuessing (t_board * b);
static INLINE void TakeNextUnexploredGuess (t_board * b);
static INLINE void RewindGuess (t_board * b);
static INLINE void RecoverPrevStateForGuessing (t_board * b);

/*///////////////////////////////////////////////////////////////// */

int Solve (t_board * b,    /* Puzzle to solve */

           char traplevel, /* 0 == never stop */
                           /* N == stop after first N-level rule */

           char bguess,    /* TRUE  == "guess if needed" */
                           /* FALSE == "do not guess" */
           volatile
           char * abort)   /* Flag to check from time to time */
{
  if (!b ||
      (b->state!=unsolved &&   /* State should be unsolved */
       b->state!=impossible))  /* or imposiible */
    return 0;

  b->ruleLevel = 0;              /* Reset rule level */

  for (;;)    /* Guessing loop (try solving first, then guess) */
  {
                  /* Solving loop (repeat until dead point) */
    for (;;)
    {
      if (abort && *abort)
        return -1;

      if (ListGetNum(&b->DirtyCL))        /* Process cells first */

        ProcessDirtyCell (b);

      else if (ListGetNum(&b->CleanGL)!=b->nGr)   /* Then groups */
      {
        if (ProcessDirtyGroup (b,
                               TRUE,       /* apply first rule */
                               FALSE,      /* optimized search */
                               abort) < 0) /* aborted -> -1 */
          return -1;

        if (traplevel>0 &&
            b->ruleLevel>=traplevel)      /* If trap "flag" says */
          return b->nGss;                 /* so, stop */
      }

      else                                /* Nothing more to do.. */
        break;                            /* ...remain unsolved */
    }

    if (b->state==unsolved &&      /* If there's no contradiction */
        b->nSol==b->nCl)           /* and all cells are solved... */
      b->state = solved;           /* ... happy end!              */

    if (b->state==solved ||        /* Solved...               end */
        !bguess ||                 /* Guess forbidden...      end */
        (b->state==impossible &&   /* Impossible, even */
         b->nGss==0))              /*      before guessing... end */

      return b->nGss;


    if (b->state==impossible)      /* If there's no solution */
    {                              /* after a guess has been */
      RewindGuess (b);             /* made, go back to try */
                                   /* other option(s) */
      if (b->nGss==0)
        return b->nGss;            /* No options remain...    end */

      RecoverPrevStateForGuessing (b);  /* Remove effects of */
                                        /* the wrong guess */
    }
                                   /* If there _might_ be a sol. */
    else  /* state==unsolved          search what to guess */

      SearchCandidateCellForGuessing (b);


    TakeNextUnexploredGuess (b);  /* Finally, guess */

    if (traplevel>0)       /* Guessing happens only when proper */
    {                      /* rules give no clue, so it is */
      b->ruleType = guess; /* somehow a "top" level rule */
      b->ruleLevel = -1;
      return b->nGss;
    }
  }
}

int FindRule (t_board * b,     /* Puzzle to solve */

              char exhaustive, /* Exhaustive or optimized search */
              
              volatile
              char * abort)    /* When true, return -1 ASAP */
{
  if (!b ||
      (b->state!=unsolved &&   /* State should be unsolved */
       b->state!=impossible))  /* or imposiible */
    return 0;

  b->ruleLevel = 0;              /* Reset rule level */

  while (ListGetNum(&b->DirtyCL))
    ProcessDirtyCell (b);

  if (b->state==unsolved &&     /* If there's no contradiction */
      b->nSol==b->nCl)          /* and all cells are solved... */
  {                             /* ... happy end!              */
    b->state = solved;
    return 0;
  }

  while (b->ruleLevel==0 &&
         ListGetNum(&b->CleanGL)!=b->nGr)
    if (ProcessDirtyGroup (b, FALSE, exhaustive, abort) < 0)
      return -1;

  return b->ruleLevel;
}

static INLINE void ProcessDirtyCell (t_board * b)
{
  int i, j, val, dirty;
  t_cell * c;
  t_group * g;
  t_lnode * node;

  node = ListGetFirst (&b->DirtyCL);      /* Take the first */
  c = GET_ELEM (t_cell, node, lnode);     /* dirty cell */

  ListExtract (&b->DirtyCL, &c->lnode);
  ListAppend (&b->CleanCL, &c->lnode);    /* Move it to the clean */
                                          /* cells list and */
  c->flags &= ~ DIRTY_FLAG;               /* mark it clean */

  if (c->nPV==0)  /* If there's no possible value at all */
  {
    b->errGr = (int)(c->ppGr[0] -
                     b->pGr);       /* Save error information */
    b->errmCl = 1 << c->pxGr[0];    /* and go on */
    b->errmSy = 0;
    b->state = impossible;          
  }
                                  /* If there's only one */
  if (c->nPV==1 &&                /* possible value... */
      !(c->flags & SOLVED_FLAG))  /* (do this only once) */
  {
    c->flags |= SOLVED_FLAG; /* This cell is solved */

    c->nSolPos = b->nSol;    /* Save pos. in solving order */
    c->nGss = b->nGss;       /* Save num. of prev. guessings */
    b->nSol ++;              /* Count solved cells */
  }

  for (i=0; i<c->nGr; i++)   /* For every group where this */
  {                          /* cell is in... */
    g = c->ppGr[i];
    j = c->pxGr[i];    /* Cell's index inside the group */

    for (val=dirty=0; val<b->nSy; val++)  /* Now remove the cell */
      if (((1<<val) & c->mRPV) &&         /* from the masks of the */
          ((1<<j) & g->pmPC[val]))        /* removed possible */
      {                                   /* values */
        g->pmPC[val] &= ~ (1<<j);
        g->pnPC[val] --;
        g->mDirtySy |= 1 << val;

        dirty = 1;       /* Bits channged, so raise the flag */
      }

    if (dirty)
    {
      g->mDirtyCl |= 1 << j;
                                                /* If any bit */
      if (!(g->flags & DIRTY_FLAG) || g->level) /* changed, mark */
      {                                         /* the group dirty */
        ListExtract ((g->flags & DIRTY_FLAG) ?  /* and move it to */
                     &b->pDirtyGL[g->level] :   /* the first dirty */
                     &b->CleanGL,               /* queue (unless */
                     &g->lnode);                /* it is already */
                                                /* there, of */
        ListAppend (b->pDirtyGL, &g->lnode);    /* course) */

        g->level = 0;
        g->flags |= DIRTY_FLAG;

        g->curRule = (e_rule)0; /* Type of rule */
        g->curMask = 1;         /* Symbols/cells mask */
        g->curMaskOnes = 1;     /* Number of ones in mask */
        g->curMaskPos = 0;      /* Pos. in current mask (MSB) */
        g->curSy = 0;           /* Symbol in rule 2 */
        g->curIGr = 0;          /* Intersected group */
      }
    }
  }

  c->mRPV = 0;   /* Clear the removed bits */
}

static INLINE int ProcessDirtyGroup (t_board * b,
                                     char apply,
                                     char exhaustive,
                                     volatile char * abort)
{
  int i, j, k, m, n, o, p, mc, ms;
  char level, sy, N, dirty, end;
  char * pInd;
  t_group * g, * Ig;
  t_cell * c;
  t_lnode * node;

  for (level=0; level<b->nSy; level++)  /* Search for the lowest */
    if (ListGetNum(b->pDirtyGL+level))  /* dirty groups list */
      break;                            /* containing groups */

  node = ListGetFirst (&b->pDirtyGL[level]);  /* Take the first */
  g = GET_ELEM (t_group, node, lnode);        /* dirty group */

  b->ruleLevel = level + 1;
  b->ruleGr = (short)(g - b->pGr);
  b->ruleIGr = -1;
  b->rulemSy = 0;


  /* ------------------ FIRST RULE ------------------ */
  /* */
  /* Part A: */
  /*   N symbols are possible in only N cells, so */
  /*   no other symbol can be possible in these cells */
  /* */
  /* Part B: */
  /*   N cells have only a set of N different possible */
  /*   symbols, so these are impossible in other cells */


  N = level + 1;   /* Just store these values in local vars */
  pInd = b->pInd;  /* so that the code bellow is more readable */

  dirty = 0;
  mc = ms = 0;

  /* Part A: (N symbols appear in only N cells..) */

  if (level<b->nSy/2 && g->mDirtySy && g->curRule==symbols)
  {
    b->ruleType = symbols;
    end = 0;

    for (i=n=0; i<b->nSy; i++)      /* Count the number of symbols */
      if (g->pnPC[i]<=N &&          /* that can appear in N (or */
          ((1<<i) & g->mDirtySy) && /* less) different cells of */
          (exhaustive || N==1 ||    /* the group and make an index */
           g->pnPC[i]>1))
      {
        pInd[n] = i;
        n ++;                     /* Take first the symbols that */
      }                           /* have been recently declared */
                                  /* impossible in dirty cells */
    for (i=0, p=n; i<b->nSy; i++) /* (dirty symbols) */
      if (g->pnPC[i]<=N &&
          !((1<<i) & g->mDirtySy) &&
          g->pnPC[i]>1)
      {
        pInd[n] = i;              /* (p: number of dirty symbols) */
        n ++;
      }

    m = g->curMask;     /* (m: subset of symbol indexes) */
    o = g->curMaskOnes; /* (o: number of ones in m) */
    k = g->curMaskPos;  /* (k: position in the mask) */

    if (n<N)
      end = 1;
    else          /* (n>=N) there are at least N such symbols */
    {
      while (!dirty)
      {
        if (abort && *abort)
          return -1;

        for (i=mc=0; i<n; i++)       /* (mc: mask indicating */
          if (m & (1<<i))            /*      subset of cells of */
            mc |= g->pmPC[pInd[i]];  /*      the group where the */
                                     /*      symbols are possible) */
        for (j=0, i=mc; i; i>>=1)
          if (i & 1)                 /* (j: number of cells) */
            j ++;
                           /* N symbols that can appear in */
        if (o==N && j<=N)  /* only N (or less) cells... */
        {
          for (i=ms=0; i<n; i++)     /* (ms: mask with N symbols) */
            if (m & (1<<i))
              ms |= 1 << pInd[i];

          if (j<N &&                 /* If N symbols can only be */
              b->state!=impossible)  /* in _less_ than N cells, */
          {                          /* at least one of them */
            b->errGr = (short)(g -   /* can't be anywhere */
                            b->pGr);
            b->errmCl = mc;
            b->errmSy = ms;       
            b->state = impossible;   /* There's no solution */
          }
                                     /* These N symbols will fill */
          for (i=0; i<b->nSy; i++)   /* the N cells, so no other */
          {                          /* symbol will be possible */
            c = g->ppCl[i];          /* in these cells */

            if (((1<<i) & mc)  &&    /* If the possible symbols */
                (~ms & c->mPV)    )  /* mask of the cell */
            {                        /* includes other symbols */

              if (apply)
                RemovePossibleValuesFromCell (b, c, ~ms);

              if (c->nPV==0 &&          /* No poss. symbol... */
                  b->state!=impossible)
              {
                b->errGr = (short)(g - b->pGr);
                b->errmCl = 1 << i;
                b->errmSy = 0;
                b->state = impossible; /* There's no solution */
              }

              dirty = 1;
            }
          }
        }
                                   /* Next subset: */
        if (o==N || j>N || k==n-1) /* If there are already N */
        {                          /* symbols, or they might be */
          m &= ~ (1<<k);           /* in more than N cells, or the */
          o --;                    /* last added is the last we */
                                   /* have... remove it */
          if (k==n-1)                 
          {                        /* If there are not more... */
            if (!o)
            {                      /* Empty hands... end */
              end = 1;
              break;
            }

            do k--; while (!(m & (1<<k))); /* Go back to the */
                                           /* previous symbol */
            m &= ~ (1<<k);                 /* and remove it */
            o --;
          }
        }
                      /* Move left and add the next symbol */
        k ++;
        m |= 1 << k;  /* Don't wait to have N symbols: go up */
        o ++;         /* and count the cells... (it saves time) */

        if (!(m & ((1<<p)-1)) ||     /* If there's no hope to have */
            !(m & ((1<<(n-N+1))-1))) /* a subset of N symbols with */
        {                            /* at least one dirty symbol, */
          end = 1;                   /* stop */
          break;
        }
      }
    }

    if (end)
    {
      g->curRule = cells;  /* Prepare for next rule */
      g->curMask = 1;
      g->curMaskOnes = 1;
      g->curMaskPos = 0;
    }
    else
    {
      g->curMask = m;     /* (m: subset of cell indexes) */
      g->curMaskOnes = o; /* (o: number of ones in m) */
      g->curMaskPos = k;  /* (k: position in the mask MSB) */
    }

    if (dirty)    /* If there are dirty cells now, */
    {
      b->rulemSy = ms;   /* Store symbols mask and */
      b->rulemCl = mc;   /* cells mask for the user */

      switch (N)
      {                                    /* Count number of */
        case 1:  b->nRulesN1 ++;   break;  /* rules applied */
        case 2:  b->nRulesN2 ++;   break;
        default: b->nRulesNGt2 ++;
      }

      return N;
    }
  }

    /* Part B (N cells can only have N symbols..) */

  if (level<b->nSy/2 && g->mDirtyCl && g->curRule==cells)
  {
    b->ruleType = cells;
    end = 0;

    for (i=n=0; i<b->nSy; i++)      /* Count the number of cells */
      if (g->ppCl[i]->nPV<=N &&     /* of the group where N (or */
          ((1<<i) & g->mDirtyCl) && /* less) different symbols */
          (exhaustive || N==1 ||    /* can be and make an index */
           g->ppCl[i]->nPV>1))
      {
        pInd[n] = i;
        n ++;
      }                           /* Take first the cells that */
                                  /* have been dirty since last */
    for (i=0, p=n; i<b->nSy; i++) /* move */
      if (g->ppCl[i]->nPV<=N &&
          !((1<<i) & g->mDirtyCl) &&
          g->ppCl[i]->nPV>1)
      {
        pInd[n] = i;
        n ++;
      }

    m = g->curMask;     /* (m: subset of cell indexes) */
    o = g->curMaskOnes; /* (o: number of ones in m) */
    k = g->curMaskPos;  /* (k: position in the mask MSB) */

    if (n<N)
      end = 1;
    else          /* (n>=N) there are at least N such cells */
    {
      while (!dirty)
      {
        if (abort && *abort)
          return -1;

        for (i=ms=0; i<n; i++)       /* (ms: mask indicating */
          if (m & (1<<i))            /*      subset of symbols */
            ms |= g->                /*      possible in the */
                 ppCl[pInd[i]]->mPV; /*      chosen cells) */

        for (j=0, i=ms; i; i>>=1)
          if (i & 1)                 /* (j: number of symbols) */
            j ++;
                             /* N cells that can have only N */
        if (o==N && j<=N)    /* (or less) different symbols... */
        {
          for (i=mc=0; i<n; i++)     /* (ms: mask with N cells) */
            if (m & (1<<i))
              mc |= 1 << pInd[i];

          if (j<N &&                 /* If the number of possible */
              b->state!=impossible)  /* symbols in N cells is */
          {                          /* _less_ than N, some cell */
            b->errGr = (short)(g -   /* can't have any symbol */
                            b->pGr);
            b->errmCl = mc;
            b->errmSy = ms;       
            b->state = impossible;   /* There's no solution */
          }
                                    /* These N symbols will fill */
          for (i=0; i<b->nSy; i++) /* the N cells, so they will */
          {                        /* not be possible in any */
            c = g->ppCl[i];        /* other cells of the group */

            if (!((1<<i) & mc) &&    /* If the possible symbols */
                (ms & c->mPV)     )  /* mask of the cell */
            {                        /* includes any one of */
                                     /* these N symbols */

              if (apply)
                RemovePossibleValuesFromCell (b, c, ms);

              if (c->nPV==0 &&          /* No poss. symbol... */
                  b->state!=impossible)
              {
                b->errGr = (short)(g - b->pGr);
                b->errmCl = 1 << i;
                b->errmSy = 0;
                b->state = impossible; /* There's no solution */
              }

              dirty = 1;
            }
          }
        }
                                   /* Next subset: */
        if (o==N || j>N || k==n-1) /* If there are already N */
        {                          /* cells, or they might have */
          m &= ~ (1<<k);           /* more than N symbols, or the */
          o --;                    /* last added is the last we */
                                   /* have... remove it */
          if (k==n-1)
          {                        /* If there are not more... */
            if (!o)
            {                      /* Empty hands... end */
              end = 1;
              break;
            }

            do k--; while (!(m & (1<<k))); /* Go back to the */
                                           /* previous cell */
            m &= ~ (1<<k);                 /* and remove it */
            o --;
          }
        }
                      /* Move left and add the next cell */
        k ++;
        m |= 1 << k;  /* Don't wait to have N cells: go up */
        o ++;         /* and count the symbols... (it saves time) */

        if (!(m & ((1<<p)-1)) ||     /* If there's no hope to have */
            !(m & ((1<<(n-N+1))-1))) /* a subset of N cells with */
        {                            /* at least one dirty cell, */
          end = 1;                   /* stop */
          break;
        }
      }
    }

    if (end)
    {
      g->curRule = inters;  /* Prepare for next rule */
      g->curSy = 0;
      g->curIGr = 0;
    }
    else
    {
      g->curMask = m;     /* (m: subset of cell indexes) */
      g->curMaskOnes = o; /* (o: number of ones in m) */
      g->curMaskPos = k;  /* (k: position in the mask MSB) */
    }

    if (dirty)    /* If there are dirty cells now, */
    {
      b->rulemSy = ms;   /* Store symbols mask and */
      b->rulemCl = mc;   /* cells mask for the user */

      switch (N)
      {                                    /* Count number of */
        case 1:  b->nRulesN1 ++;   break;  /* rules applied */
        case 2:  b->nRulesN2 ++;   break;
        default: b->nRulesNGt2 ++;
      }

      return N;
    }
  }


  /* ------------------ SECOND RULE ------------------ */
  /* */
  /*   A symbol is only possible N cells, and they all */
  /*   belong to the intersection with another group, */
  /*   so it will not appear anywhere else in that */
  /*   other group */


  b->ruleType = inters;
                         /* Check only sets of cells that can be */
  if (level<b->maxICl && /* all involved in the same intersection */
      level>0 &&         /* Case N==1 will be detected by rule 1 */
      g->curRule==inters)
  {
    for (sy=g->curSy;              /* For every symbol that */
         sy<b->nSy;                /* can be in exactly */
         sy++, g->curIGr=0)        /* level+1 cells in g... */
      if (g->pnPC[sy]==level+1)
      {
        if (abort && *abort)
          return -1;

        for (i=g->curIGr;
             i<g->nIGr; i++)       /* For every intersected group */
          if ((~g->pmICl[i] &      /* if all those cells of g are */
               g->pmPC[sy]   )==0) /* involved, then the symbol */
          {                        /* can not appear anywhere */
            Ig = g->ppIGr[i];      /* else in the intersected */
            j = g->pxIGr[i];       /* but in those involved cells */

            if (~Ig->pmICl[j] &         /* So, if it's */
                Ig->pmPC[sy]   )        /* marked possible... */
            {
              for (k=mc=0; k<b->nSy; k++)         /* For those */
                if ( (Ig->pmPC[sy] & (1<<k)) &&   /* cells not */
                    !(Ig->pmICl[j] & (1<<k))   )  /* involved... */
                {
                  c = Ig->ppCl[k];

                  if (c->mPV & (1<<sy))   /* If the cell has that */
                  {                       /* symbol as possible.. */

                    if (apply)
                      RemovePossibleValuesFromCell (b, c, 1<<sy);

                    mc |= 1 << k;

                    if (c->nPV==0 &&          /* No poss. symbol */
                        b->state!=impossible)
                    {
                      b->errGr = (short)(Ig - b->pGr);
                      b->errmCl = 1 << k;
                      b->errmSy = 0;
                      b->state = impossible; /* There's no sol. */
                    }
                  }
                }

              if (!Ig->pmPC[sy] &&           /* If the symbol is */
                  b->state!=impossible)      /* impossible now in */
              {                              /* the group Ig, */
                b->errGr = (short)(Ig -      /* save error info. */
                                   b->pGr);  
                b->errmCl = 0;
                b->errmSy = 0;
                b->state = impossible; /* There's no solution */
              }

              switch (N)
              {                                   /* Count number */
                case 1:  b->nRulesN1 ++;   break; /* of rules */
                case 2:  b->nRulesN2 ++;   break; /* applied */
                default: b->nRulesNGt2 ++;
              }

              b->ruleIGr = (short)(Ig - /* Store rule information */
                               b->pGr); /* for the user: masks */
              b->rulemSy = 1 << sy;     /* and intersected group */
              b->rulemCl = mc;          /* (mc refers to it) */

              g->curSy = sy;
              g->curIGr = i+1;

              return N;   /* Go out (there are dirty cells now..) */
            }
          }
      }

    g->curRule = norule;
  }

  /* ------------ NO MORE RULES */
                                        /* Nothing changed, so */
                                        /* the group is "clean" */
  ListExtract (b->pDirtyGL+level,       /* (at this level) */
               &g->lnode);

  g->curRule = (e_rule)0; /* Type of rule */
  g->curMask = 1;         /* Symbols/cells mask */
  g->curMaskOnes = 1;     /* Number of ones in mask */
  g->curMaskPos = 0;      /* Pos. in current mask (MSB) */
  g->curSy = 0;           /* Symbol in rule 2 */
  g->curIGr = 0;          /* Intersected group */

  g->level ++;                          /* Extract it from this */
                                        /* dirty groups list */
  if (g->level<b->maxLevel)             /* and append it to the */
    ListAppend (b->pDirtyGL+g->level,   /* next level list, or */
                &g->lnode);             /* the clean groups list */
  else
  {
    ListAppend (&b->CleanGL, &g->lnode);
    g->flags &= ~ DIRTY_FLAG;
    g->mDirtyCl = 0;
    g->mDirtySy = 0;
  }

  b->ruleLevel = 0;  /* Rules had no effects */
  return 0;
}

static INLINE void RemovePossibleValuesFromCell (t_board * b,
                                                 t_cell * c,
                                                 int m)
{
  int i, n, sy;

  c->mRPV |= m & c->mPV;         /* Backup ones to be removed */
  c->mPV &= ~m;                  /* and then remove them */

  for (i=n=0, sy=-1; i<b->nSy; i++)
    if (c->mPV & (1<<i))
    {
      n ++;        /* Count remaining possible values */
      sy = i;      /* and remember one of them */
    }

  c->nPV = n;

  if (n==1)        /* If only one symbol is possible now, */
    c->FV = sy;    /* save it */

  if ( ! (c->flags & DIRTY_FLAG)) /* If the cell */
  {                               /* is clean, */
    ListExtract (&b->CleanCL,     /* move it to */
                 &c->lnode);      /* the dirty */
                                  /* cells list */
    ListInsert (&b->DirtyCL,
                &c->lnode);

    c->flags |= DIRTY_FLAG;
  }
}

static INLINE void SearchCandidateCellForGuessing (t_board * b)
{
  char min;
  t_cell * c, * cand;

  for (c=cand=b->pCl, min=b->nSy+1; c<b->pCl+b->nCl; c++)
    if (c->nPV==2)
    {                          /* Search for the cell with less */
      cand = c;                /* possible values above 1 */
      break;                   /* (2 PV is perfect) */
    }
    else if (c->nPV>1 && c->nPV<min)
    {
      cand = c;
      min = c->nPV;
    }

  cand->flags |= GUESSED_FLAG;   /* Mark the chosen cell guessed */

  b->pnGssCl[b->nGss] = (short)(cand - b->pCl); /* Cell number */
  b->pmPGss[b->nGss] = cand->mPV;               /* Poss. values */
                                                /* to explore */
  b->nGss ++;    /* Push */
}

static INLINE void TakeNextUnexploredGuess (t_board * b)
{
  int i;
  t_cell * c;

  for (i=0; i<b->nSy; i++)              /* Search for the next */
    if (b->pmPGss[b->nGss-1] & (1<<i))  /* value to guess */
      break;
                                        /* Remove it from the */
  b->pmPGss[b->nGss-1] &= ~ (1<<i);     /* pending values mask */

  c = b->pCl + b->pnGssCl[b->nGss-1];   /* Take the cell, and */
                                         /* guess the chosen */
  RemovePossibleValuesFromCell (b, c,     /* value (remove the */
                                ~(1<<i));  /* rest of values) */

  b->nGssTotal ++;
}

static INLINE void RewindGuess (t_board * b)
{
  t_cell * c;
                                  /* Go back through last guess */
  while (b->nGss>0 &&             /* operations searching for an */
         !b->pmPGss[b->nGss-1])   /* unexplored option */
  {
    b->nGss --;                        /* Pop last guess */

    c = b->pCl + b->pnGssCl[b->nGss];  /* Remvove the guessed */
    c->flags &= ~ GUESSED_FLAG;        /* mark from the cell */
  }
}

static INLINE void RecoverPrevStateForGuessing (t_board * b)
{
  int i, n, mv;
  t_cell * c, * gc;
  t_group * g;

  n = b->nSy;           /* Number of symbols */
  mv = ~ (~0UL << n);   /* Mask with all of them */

  gc = b->pCl + b->pnGssCl[b->nGss-1];  /* Current guessed cell */

  b->state = unsolved;  /* Restore state */

  for (c=b->pCl; c<b->pCl+b->nCl; c++)  /* For every cell */
  {
    c->mPV = mv;                    /* First, add all symbols */
    c->nPV = n;                     /* as possible values */
    c->mRPV = 0;

    if (c!=gc &&                /* For all the cells guessed */
        (c->flags &             /* or solved _before_ current */
         SOLVED_FLAG) &&        /* guess, set their original */
        ((c->flags &            /* value, marking them dirty */
          GUESSED_FLAG) ||      
         c->nGss<b->nGss))    

      RemovePossibleValuesFromCell (b, c, ~(1<<c->FV));

    else if (c->flags & SOLVED_FLAG)
    {
      c->flags &= ~ SOLVED_FLAG;
      b->nSol --;
    }
  }

  for (g=b->pGr; g<b->pGr+b->nGr; g++)  /* For every group */
    for (i=0; i<n; i++)                 /* and every value */
    {
      g->pmPC[i] = mv;    /* Mark all the cells of the group */
      g->pnPC[i] = n;     /* as possible */

      g->mDirtyCl = 0;
      g->mDirtySy = 0;
    }
}
