/*
-------------------------------------------------------------------
SUDOKU SENSEI 2.00: a Sudoku Explainer Application
Copyright (C) 2007  Victor Antolin Urtueta

This program is based on SUDOKU SENSEI 1.04, a Sudoku Explainer
Engine by Martin Knoblauch.

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
MA  02110-1301, USA.

Contact the authors: mentur.SudokuSensei AT gmail.com
                     comocomocomo AT users.sourceforge.net
Latest versions: http://sourceforge.net/projects/sudoku-sensei
-------------------------------------------------------------------
*/
/************************************************************************************************/
/*  File: thread-sensei.h    * Status: unstable * Version: 1.0 * finished in version: 0.45 	*/
/************************************************************************************************/
/*        									      		*/
/*  This file contains the definitions of funtions of SenseiThread class, and the definition of	*/
/*  s_rule_node struct										*/
/*        									      		*/
/************************************************************************************************/

#include "thread-sensei.h"
#include "dmem.h"
#include "myassert.h"

//#define _DEBUG
#define ERROR_NOT_INITIALIZED 1

struct s_rule_node
{
  s_rule rule;
  s_rule_node * next;

  s_rule_node (const s_rule & r)
    : rule(r), next(NULL) {}
};

int SenseiThread::DumpRules ()
{
  int n;
  s_rule_node * p;

  prot_rules_mutex.lock ();

  for (n=0; first; n++)
  {
    rules.append (first->rule);
    p = first;
    first = first->next;
    delete DM_DEL (p);
  }

  prot_rules_mutex.unlock ();

  return n;
}

void SenseiThread::Clear ()
{
  s_rule_node * p;

  prot_rules_mutex.lock ();

  while (first)
  {
    p = first;
    first = first->next;
    delete DM_DEL (p);
  }

  rules.clear ();

  prot_rules_mutex.unlock ();
}

void SenseiThread::Init (t_board_p pBoard,
                         bool bOnlyIntegrity)
{
  work_board = pBoard;
  flagOnlyCheckIntegrity = bOnlyIntegrity;

  MY_ASSERT (threadState==not_running);

  if (pBoard)
  {
    flagCancelOperations = 0;
    Clear ();
  }
  else
    flagCancelOperations = 1;
}

void SenseiThread::AppendRule (const s_rule & r)
{
  MY_ASSERT (threadState==running);

  prot_rules_mutex.lock ();

  s_rule_node * p = DM_NEW( new  s_rule_node(r) );

  if (first)
    last = last->next = p;
  else
    first = last = p;

  prot_rules_mutex.unlock ();

  emit NewRule ();
}

void SenseiThread::run ()
{
  e_state state;
  int guess;
  s_rule r;

  threadState = running;

  if (!work_board || flagCancelOperations)
  {
    emit Error (ERROR_NOT_INITIALIZED);
    flagCancelOperations = 1;
    threadState = not_running;
    return;
  }

  if (flagOnlyCheckIntegrity)
  {

#ifdef _DEBUG
	printf ("----Thread Sensei: Run IntegrityChecked 1--\n");
#endif

    guess = Solve (work_board,
                   0,                       // No pause
                   1,                       // Guess if necessary
                   &flagCancelOperations);

    state = GetState (work_board);

    if (guess<0 || flagCancelOperations)
    {
      emit Aborted ();

#ifdef _DEBUG
	printf ("----Thread Sensei: Run IntegrityChecked ENDING ABORTED--\n");
#endif
    }
    else
    {
      emit IntegrityChecked (state==solved ? guess : -1);

#ifdef _DEBUG
	printf ("----Thread Sensei: Run IntegrityChecked ENDING--\n");
#endif
    }

    flagCancelOperations = 1;
    threadState = not_running;
    return;
  }

#ifdef _DEBUG
	printf ("----Thread Sensei: Run Rules 1--\n");
#endif
  for (;;)
  {
    if (flagCancelOperations)
    {
      emit Aborted ();
#ifdef _DEBUG
	printf ("----Thread Sensei: Run Rules ENDING ABORTED--\n");
#endif
      threadState = not_running;
      return;
    }

    if (FindRule(work_board,
                 TRUE,                         //-- TRUE == exhaustive
                 &flagCancelOperations) <= 0)
    {
      emit NoMoreRulestoFind ();
      flagCancelOperations = 1;


#ifdef _DEBUG
	printf ("----Thread Sensei: Run Rules ENDING--\n");
#endif

      threadState = not_running;
      return;
    }

    r.type = GetLastRuleType (work_board);
    r.level = GetLastRuleLevel (work_board);
    r.group = GetLastRuleGroup (work_board);
    r.inters_group = GetLastRuleIntersectedGroup (work_board);
    r.cells_mask = GetLastRuleCellsMask (work_board);
    r.symbols_mask = GetLastRuleSymbolsMask (work_board);

    AppendRule (r);
  }
}
